@@ -5,22 +5,27 @@
 
     LaTeX builder.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import os
 from os import path
 
+from six import iteritems
 from docutils import nodes
 from docutils.io import FileOutput
 from docutils.utils import new_document
 from docutils.frontend import OptionParser
 
 from sphinx import package_dir, addnodes
-from sphinx.util import SEP, texescape, copyfile
+from sphinx.util import texescape
+from sphinx.errors import SphinxError
+from sphinx.locale import _
 from sphinx.builders import Builder
 from sphinx.environment import NoUri
+from sphinx.util.nodes import inline_all_toctrees
+from sphinx.util.osutil import SEP, copyfile
 from sphinx.util.console import bold, darkgreen
 from sphinx.writers.latex import LaTeXWriter
 
@@ -33,6 +38,7 @@ class LaTeXBuilder(Builder):
     format = 'latex'
     supported_image_types = ['application/pdf', 'image/png',
                              'image/gif', 'image/jpeg']
+    usepackages = []
 
     def init(self):
         self.docnames = []
@@ -40,13 +46,9 @@ class LaTeXBuilder(Builder):
         texescape.init()
 
     def get_outdated_docs(self):
-        return 'all documents' # for now
+        return 'all documents'  # for now
 
     def get_target_uri(self, docname, typ=None):
-        if typ == 'token':
-            # token references are always inside production lists and must be
-            # replaced by \token{} in LaTeX
-            return '@token'
         if docname not in self.docnames:
             raise NoUri
         else:
@@ -57,7 +59,7 @@ class LaTeXBuilder(Builder):
         return self.get_target_uri(to, typ)
 
     def init_document_data(self):
-        preliminary_document_data = map(list, self.config.latex_documents)
+        preliminary_document_data = [list(x) for x in self.config.latex_documents]
         if not preliminary_document_data:
             self.warn('no "latex_documents" config value found; no documents '
                       'will be written')
@@ -79,7 +81,8 @@ class LaTeXBuilder(Builder):
         docwriter = LaTeXWriter(self)
         docsettings = OptionParser(
             defaults=self.env.settings,
-            components=(docwriter,)).get_default_values()
+            components=(docwriter,),
+            read_config_files=True).get_default_values()
 
         self.init_document_data()
 
@@ -92,49 +95,42 @@ class LaTeXBuilder(Builder):
                 destination_path=path.join(self.outdir, targetname),
                 encoding='utf-8')
             self.info("processing " + targetname + "... ", nonl=1)
-            doctree = self.assemble_doctree(docname, toctree_only,
-                appendices=((docclass == 'manual') and
-                            self.config.latex_appendices or []))
+            toctrees = self.env.get_doctree(docname).traverse(addnodes.toctree)
+            if toctrees:
+                if toctrees[0].get('maxdepth') > 0:
+                    tocdepth = toctrees[0].get('maxdepth')
+                else:
+                    tocdepth = None
+            else:
+                tocdepth = None
+            doctree = self.assemble_doctree(
+                docname, toctree_only,
+                appendices=((docclass != 'howto') and self.config.latex_appendices or []))
+            doctree['tocdepth'] = tocdepth
             self.post_process_images(doctree)
             self.info("writing... ", nonl=1)
             doctree.settings = docsettings
             doctree.settings.author = author
             doctree.settings.title = title
+            doctree.settings.contentsname = self.get_contentsname(docname)
             doctree.settings.docname = docname
             doctree.settings.docclass = docclass
-            try:
-                docwriter.write(doctree, destination)
-            except UnicodeError:
-                self.warn("a Unicode error occurred when writing the output. "
-                          "Please make sure all config values that contain "
-                          "non-ASCII content are Unicode strings.")
-                return
+            docwriter.write(doctree, destination)
             self.info("done")
 
+    def get_contentsname(self, indexfile):
+        tree = self.env.get_doctree(indexfile)
+        contentsname = None
+        for toctree in tree.traverse(addnodes.toctree):
+            if toctree['caption']:
+                contentsname = toctree['caption']
+                break
+
+        return contentsname
+
     def assemble_doctree(self, indexfile, toctree_only, appendices):
         self.docnames = set([indexfile] + appendices)
         self.info(darkgreen(indexfile) + " ", nonl=1)
-        def process_tree(docname, tree):
-            tree = tree.deepcopy()
-            for toctreenode in tree.traverse(addnodes.toctree):
-                newnodes = []
-                includefiles = map(str, toctreenode['includefiles'])
-                for includefile in includefiles:
-                    try:
-                        self.info(darkgreen(includefile) + " ", nonl=1)
-                        subtree = process_tree(
-                            includefile, self.env.get_doctree(includefile))
-                        self.docnames.add(includefile)
-                    except Exception:
-                        self.warn('toctree contains ref to nonexisting '
-                                  'file %r' % includefile,
-                                  self.env.doc2path(docname))
-                    else:
-                        sof = addnodes.start_of_file(docname=includefile)
-                        sof.children = subtree.children
-                        newnodes.append(sof)
-                toctreenode.parent.replace(toctreenode, newnodes)
-            return tree
         tree = self.env.get_doctree(indexfile)
         tree['docname'] = indexfile
         if toctree_only:
@@ -148,7 +144,8 @@ class LaTeXBuilder(Builder):
             for node in tree.traverse(addnodes.toctree):
                 new_sect += node
             tree = new_tree
-        largetree = process_tree(indexfile, tree)
+        largetree = inline_all_toctrees(self, self.docnames, indexfile, tree,
+                                        darkgreen, [indexfile])
         largetree['docname'] = indexfile
         for docname in appendices:
             appendix = self.env.get_doctree(docname)
@@ -178,12 +175,20 @@ class LaTeXBuilder(Builder):
         # copy image files
         if self.images:
             self.info(bold('copying images...'), nonl=1)
-            for src, dest in self.images.iteritems():
+            for src, dest in iteritems(self.images):
                 self.info(' '+src, nonl=1)
                 copyfile(path.join(self.srcdir, src),
                          path.join(self.outdir, dest))
             self.info()
 
+        # copy TeX support files from texinputs
+        self.info(bold('copying TeX support files...'))
+        staticdirname = path.join(package_dir, 'texinputs')
+        for filename in os.listdir(staticdirname):
+            if not filename.startswith('.'):
+                copyfile(path.join(staticdirname, filename),
+                         path.join(self.outdir, filename))
+
         # copy additional files
         if self.config.latex_additional_files:
             self.info(bold('copying additional files...'), nonl=1)
@@ -196,13 +201,9 @@ class LaTeXBuilder(Builder):
         # the logo is handled differently
         if self.config.latex_logo:
             logobase = path.basename(self.config.latex_logo)
-            copyfile(path.join(self.confdir, self.config.latex_logo),
-                     path.join(self.outdir, logobase))
-
-        self.info(bold('copying TeX support files... '), nonl=True)
-        staticdirname = path.join(package_dir, 'texinputs')
-        for filename in os.listdir(staticdirname):
-            if not filename.startswith('.'):
-                copyfile(path.join(staticdirname, filename),
-                         path.join(self.outdir, filename))
+            logotarget = path.join(self.outdir, logobase)
+            if not path.isfile(path.join(self.confdir, self.config.latex_logo)):
+                raise SphinxError('logo file %r does not exist' % self.config.latex_logo)
+            elif not path.isfile(logotarget):
+                copyfile(path.join(self.confdir, self.config.latex_logo), logotarget)
         self.info('done')
