@@ -5,7 +5,7 @@
 
     Build input files for the Qt collection generator.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,6 +15,7 @@ import codecs
 import posixpath
 from os import path
 
+from six import text_type
 from docutils import nodes
 
 from sphinx import addnodes
@@ -110,7 +111,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         StandaloneHTMLBuilder.init(self)
         # the output files for HTML help must be .html only
         self.out_suffix = '.html'
-        #self.config.html_style = 'traditional.css'
+        # self.config.html_style = 'traditional.css'
 
     def handle_finish(self):
         self.build_qhp(self.outdir, self.config.qthelp_basename)
@@ -121,9 +122,10 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         # sections
         tocdoc = self.env.get_and_resolve_doctree(self.config.master_doc, self,
                                                   prune_toctrees=False)
-        istoctree = lambda node: (
-                        isinstance(node, addnodes.compact_paragraph)
-                            and node.has_key('toctree'))
+
+        def istoctree(node):
+            return isinstance(node, addnodes.compact_paragraph) and \
+                'toctree' in node
         sections = []
         for node in tocdoc.traverse(istoctree):
             sections.extend(self.write_toc(node))
@@ -136,7 +138,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         # they are all unicode strings before joining them
         new_sections = []
         for section in sections:
-            if not isinstance(section, unicode):
+            if not isinstance(section, text_type):
                 new_sections.append(force_decode(section, None))
             else:
                 new_sections.append(section)
@@ -156,13 +158,13 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         olen = len(outdir)
         projectfiles = []
         staticdir = path.join(outdir, '_static')
-        imagesdir = path.join(outdir, '_images')
+        imagesdir = path.join(outdir, self.imagedir)
         for root, dirs, files in os.walk(outdir):
             resourcedir = root.startswith(staticdir) or \
-                          root.startswith(imagesdir)
+                root.startswith(imagesdir)
             for fn in files:
                 if (resourcedir and not fn.endswith('.js')) or \
-                       fn.endswith('.html'):
+                   fn.endswith('.html'):
                     filename = path.join(root, fn)[olen:]
                     projectfiles.append(file_template %
                                         {'filename': htmlescape(filename)})
@@ -238,7 +240,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
                 parts.extend(self.write_toc(subnode, indentlevel))
         elif isinstance(node, nodes.reference):
             link = node['refuri']
-            title = htmlescape(node.astext()).replace('"','&quot;')
+            title = htmlescape(node.astext()).replace('"', '&quot;')
             item = section_template % {'title': title, 'ref': link}
             item = u' ' * 4 * indentlevel + item
             parts.append(item.encode('ascii', 'xmlcharrefreplace'))
@@ -257,7 +259,7 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
             groupdict = matchobj.groupdict()
             shortname = groupdict['title']
             id = groupdict.get('id')
-            #descr = groupdict.get('descr')
+            # descr = groupdict.get('descr')
             if shortname.endswith('()'):
                 shortname = shortname[:-2]
             id = '%s.%s' % (id, shortname)
@@ -276,17 +278,17 @@ class QtHelpBuilder(StandaloneHTMLBuilder):
         keywords = []
 
         title = htmlescape(title)
-#        if len(refs) == 0: # XXX
-#            write_param('See Also', title)
+        # if len(refs) == 0: # XXX
+        #     write_param('See Also', title)
         if len(refs) == 1:
             keywords.append(self.keyword_item(title, refs[0]))
         elif len(refs) > 1:
             for i, ref in enumerate(refs):  # XXX
-#                item = (' '*12 +
-#                        '<keyword name="%s [%d]" ref="%s"/>' % (
-#                                                        title, i, ref))
-#                item.encode('ascii', 'xmlcharrefreplace')
-#                keywords.append(item)
+                # item = (' '*12 +
+                #         '<keyword name="%s [%d]" ref="%s"/>' % (
+                #          title, i, ref))
+                # item.encode('ascii', 'xmlcharrefreplace')
+                # keywords.append(item)
                 keywords.append(self.keyword_item(title, ref))
 
         if subitems:
