@@ -6,7 +6,7 @@
     Allow graphviz-formatted graphs to be included in Sphinx-generated
     documents inline.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -14,16 +14,15 @@ import re
 import codecs
 import posixpath
 from os import path
-from math import ceil
 from subprocess import Popen, PIPE
-try:
-    from hashlib import sha1 as sha
-except ImportError:
-    from sha import sha
+from hashlib import sha1
 
+from six import text_type
 from docutils import nodes
 from docutils.parsers.rst import directives
+from docutils.statemachine import ViewList
 
+import sphinx
 from sphinx.errors import SphinxError
 from sphinx.locale import _
 from sphinx.util.osutil import ensuredir, ENOENT, EPIPE, EINVAL
@@ -41,6 +40,20 @@ class graphviz(nodes.General, nodes.Element):
     pass
 
 
+def figure_wrapper(directive, node, caption):
+    figure_node = nodes.figure('', node)
+
+    parsed = nodes.Element()
+    directive.state.nested_parse(ViewList([caption], source=''),
+                                 directive.content_offset, parsed)
+    caption_node = nodes.caption(parsed[0].rawsource, '',
+                                 *parsed[0].children)
+    caption_node.source = parsed[0].source
+    caption_node.line = parsed[0].line
+    figure_node += caption_node
+    return figure_node
+
+
 class Graphviz(Directive):
     """
     Directive to insert arbitrary dot markup.
@@ -86,9 +99,12 @@ class Graphviz(Directive):
         node['options'] = []
         if 'alt' in self.options:
             node['alt'] = self.options['alt']
-        if 'caption' in self.options:
-            node['caption'] = self.options['caption']
         node['inline'] = 'inline' in self.options
+
+        caption = self.options.get('caption')
+        if caption and not node['inline']:
+            node = figure_wrapper(self, node, caption)
+
         return [node]
 
 
@@ -113,26 +129,24 @@ class GraphvizSimple(Directive):
         node['options'] = []
         if 'alt' in self.options:
             node['alt'] = self.options['alt']
-        if 'caption' in self.options:
-            node['caption'] = self.options['caption']
         node['inline'] = 'inline' in self.options
+
+        caption = self.options.get('caption')
+        if caption and not node['inline']:
+            node = figure_wrapper(self, node, caption)
+
         return [node]
 
 
 def render_dot(self, code, options, format, prefix='graphviz'):
     """Render graphviz code into a PNG or PDF output file."""
-    hashkey = code.encode('utf-8') + str(options) + \
-              str(self.builder.config.graphviz_dot) + \
-              str(self.builder.config.graphviz_dot_args)
-    fname = '%s-%s.%s' % (prefix, sha(hashkey).hexdigest(), format)
-    if hasattr(self.builder, 'imgpath'):
-        # HTML
-        relfn = posixpath.join(self.builder.imgpath, fname)
-        outfn = path.join(self.builder.outdir, '_images', fname)
-    else:
-        # LaTeX
-        relfn = fname
-        outfn = path.join(self.builder.outdir, fname)
+    hashkey = (code + str(options) +
+               str(self.builder.config.graphviz_dot) +
+               str(self.builder.config.graphviz_dot_args)).encode('utf-8')
+
+    fname = '%s-%s.%s' % (prefix, sha1(hashkey).hexdigest(), format)
+    relfn = posixpath.join(self.builder.imgpath, fname)
+    outfn = path.join(self.builder.outdir, self.builder.imagedir, fname)
 
     if path.isfile(outfn):
         return relfn, outfn
@@ -144,7 +158,7 @@ def render_dot(self, code, options, format, prefix='graphviz'):
     ensuredir(path.dirname(outfn))
 
     # graphviz expects UTF-8 by default
-    if isinstance(code, unicode):
+    if isinstance(code, text_type):
         code = code.encode('utf-8')
 
     dot_args = [self.builder.config.graphviz_dot]
@@ -155,7 +169,7 @@ def render_dot(self, code, options, format, prefix='graphviz'):
         dot_args.extend(['-Tcmapx', '-o%s.map' % outfn])
     try:
         p = Popen(dot_args, stdout=PIPE, stdin=PIPE, stderr=PIPE)
-    except OSError, err:
+    except OSError as err:
         if err.errno != ENOENT:   # No such file or directory
             raise
         self.builder.warn('dot command %r cannot be run (needed for graphviz '
@@ -163,20 +177,13 @@ def render_dot(self, code, options, format, prefix='graphviz'):
                           self.builder.config.graphviz_dot)
         self.builder._graphviz_warned_dot = True
         return None, None
-    wentWrong = False
     try:
         # Graphviz may close standard input when an error occurs,
         # resulting in a broken pipe on communicate()
         stdout, stderr = p.communicate(code)
-    except (OSError, IOError), err:
-        if err.errno != EPIPE:
+    except (OSError, IOError) as err:
+        if err.errno not in (EPIPE, EINVAL):
             raise
-        wentWrong = True
-    except IOError, err:
-        if err.errno != EINVAL:
-            raise
-        wentWrong = True
-    if wentWrong:
         # in this case, read the standard output and standard error streams
         # directly, to get the error message(s)
         stdout, stderr = p.stdout.read(), p.stderr.read()
@@ -184,6 +191,9 @@ def render_dot(self, code, options, format, prefix='graphviz'):
     if p.returncode != 0:
         raise GraphvizError('dot exited with error:\n[stderr]\n%s\n'
                             '[stdout]\n%s' % (stderr, stdout))
+    if not path.isfile(outfn):
+        raise GraphvizError('dot did not produce an output file:\n[stderr]\n%s\n'
+                            '[stdout]\n%s' % (stderr, stdout))
     return relfn, outfn
 
 
@@ -195,7 +205,7 @@ def render_dot_html(self, node, code, options, prefix='graphviz',
             raise GraphvizError("graphviz_output_format must be one of 'png', "
                                 "'svg', but is %r" % format)
         fname, outfn = render_dot(self, code, options, format, prefix)
-    except GraphvizError, exc:
+    except GraphvizError as exc:
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
 
@@ -213,7 +223,8 @@ def render_dot_html(self, node, code, options, prefix='graphviz',
             alt = node.get('alt', self.encode(code).strip())
         imgcss = imgcls and 'class="%s"' % imgcls or ''
         if format == 'svg':
-            svgtag = '<img src="%s" alt="%s" %s/>\n' % (fname, alt, imgcss)
+            svgtag = '''<object data="%s" type="image/svg+xml">
+            <p class="warning">%s</p></object>\n''' % (fname, alt)
             self.body.append(svgtag)
         else:
             mapfile = open(outfn + '.map', 'rb')
@@ -227,13 +238,10 @@ def render_dot_html(self, node, code, options, prefix='graphviz',
                                  (fname, alt, imgcss))
             else:
                 # has a map: get the name of the map and connect the parts
-                mapname = mapname_re.match(imgmap[0]).group(1)
+                mapname = mapname_re.match(imgmap[0].decode('utf-8')).group(1)
                 self.body.append('<img src="%s" alt="%s" usemap="#%s" %s/>\n' %
                                  (fname, alt, mapname, imgcss))
-                self.body.extend(imgmap)
-        if node.get('caption') and not inline:
-            self.body.append('</p>\n<p class="caption">')
-            self.body.append(self.encode(node['caption']))
+                self.body.extend([item.decode('utf-8') for item in imgmap])
 
     self.body.append('</%s>\n' % wrapper)
     raise nodes.SkipNode
@@ -246,7 +254,7 @@ def html_visit_graphviz(self, node):
 def render_dot_latex(self, node, code, options, prefix='graphviz'):
     try:
         fname, outfn = render_dot(self, code, options, 'pdf', prefix)
-    except GraphvizError, exc:
+    except GraphvizError as exc:
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
 
@@ -257,18 +265,8 @@ def render_dot_latex(self, node, code, options, prefix='graphviz'):
         para_separator = '\n'
 
     if fname is not None:
-        caption = node.get('caption')
-        # XXX add ids from previous target node
-        if caption and not inline:
-            self.body.append('\n\\begin{figure}[h!]')
-            self.body.append('\n\\begin{center}')
-            self.body.append('\n\\caption{%s}' % self.encode(caption))
-            self.body.append('\n\\includegraphics{%s}' % fname)
-            self.body.append('\n\\end{center}')
-            self.body.append('\n\\end{figure}\n')
-        else:
-            self.body.append('%s\\includegraphics{%s}%s' %
-                             (para_separator, fname, para_separator))
+        self.body.append('%s\\includegraphics{%s}%s' %
+                         (para_separator, fname, para_separator))
     raise nodes.SkipNode
 
 
@@ -279,18 +277,14 @@ def latex_visit_graphviz(self, node):
 def render_dot_texinfo(self, node, code, options, prefix='graphviz'):
     try:
         fname, outfn = render_dot(self, code, options, 'png', prefix)
-    except GraphvizError, exc:
+    except GraphvizError as exc:
         self.builder.warn('dot code %r: ' % code + str(exc))
         raise nodes.SkipNode
     if fname is not None:
-        self.body.append('\n\n@float\n')
-        caption = node.get('caption')
-        if caption:
-            self.body.append('@caption{%s}\n' % self.escape_arg(caption))
-        self.body.append('@image{%s,,,[graphviz],png}\n'
-                         '@end float\n\n' % fname[:-4])
+        self.body.append('@image{%s,,,[graphviz],png}\n' % fname[:-4])
     raise nodes.SkipNode
 
+
 def texinfo_visit_graphviz(self, node):
     render_dot_texinfo(self, node, node['code'], node['options'])
 
@@ -298,13 +292,16 @@ def texinfo_visit_graphviz(self, node):
 def text_visit_graphviz(self, node):
     if 'alt' in node.attributes:
         self.add_text(_('[graph: %s]') % node['alt'])
-    self.add_text(_('[graph]'))
+    else:
+        self.add_text(_('[graph]'))
+    raise nodes.SkipNode
 
 
 def man_visit_graphviz(self, node):
     if 'alt' in node.attributes:
-        self.body.append(_('[graph: %s]') % node['alt'] + '\n')
-    self.body.append(_('[graph]'))
+        self.body.append(_('[graph: %s]') % node['alt'])
+    else:
+        self.body.append(_('[graph]'))
     raise nodes.SkipNode
 
 
@@ -321,3 +318,4 @@ def setup(app):
     app.add_config_value('graphviz_dot', 'dot', 'html')
     app.add_config_value('graphviz_dot_args', [], 'html')
     app.add_config_value('graphviz_output_format', 'png', 'html')
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
