@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""
+r"""
     sphinx.ext.inheritance_diagram
     ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -32,7 +32,7 @@
     The graph is inserted as a PNG+image map into HTML and a PDF in
     LaTeX.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -44,11 +44,17 @@ try:
 except ImportError:
     from md5 import md5
 
+from six import text_type
+from six.moves import builtins
+
 from docutils import nodes
 from docutils.parsers.rst import directives
 
-from sphinx.roles import xfileref_role
-from sphinx.ext.graphviz import render_dot_html, render_dot_latex
+import sphinx
+from sphinx.ext.graphviz import render_dot_html, render_dot_latex, \
+    render_dot_texinfo
+from sphinx.pycode import ModuleAnalyzer
+from sphinx.util import force_decode
 from sphinx.util.compat import Directive
 
 
@@ -67,28 +73,26 @@ class InheritanceGraph(object):
     from all the way to the root "object", and then is able to generate a
     graphviz dot graph from them.
     """
-    def __init__(self, class_names, currmodule, show_builtins=False):
-        """
-        *class_names* is a list of child classes to show bases from.
+    def __init__(self, class_names, currmodule, show_builtins=False,
+                 private_bases=False, parts=0):
+        """*class_names* is a list of child classes to show bases from.
 
         If *show_builtins* is True, then Python builtins will be shown
         in the graph.
         """
         self.class_names = class_names
-        self.classes = self._import_classes(class_names, currmodule)
-        self.all_classes = self._all_classes(self.classes)
-        if len(self.all_classes) == 0:
+        classes = self._import_classes(class_names, currmodule)
+        self.class_info = self._class_info(classes, show_builtins,
+                                           private_bases, parts)
+        if not self.class_info:
             raise InheritanceException('No classes found for '
                                        'inheritance diagram')
-        self.show_builtins = show_builtins
 
     def _import_class_or_module(self, name, currmodule):
-        """
-        Import a class using its fully-qualified *name*.
-        """
+        """Import a class using its fully-qualified *name*."""
         try:
             path, base = class_sig_re.match(name).groups()
-        except ValueError:
+        except (AttributeError, ValueError):
             raise InheritanceException('Invalid class or module %r specified '
                                        'for inheritance diagram' % name)
 
@@ -97,7 +101,7 @@ class InheritanceGraph(object):
 
         # two possibilities: either it is a module, then import it
         try:
-            module = __import__(fullname)
+            __import__(fullname)
             todoc = sys.modules[fullname]
         except ImportError:
             # else it is a class, then import the module
@@ -110,7 +114,7 @@ class InheritanceGraph(object):
                         'Could not import class %r specified for '
                         'inheritance diagram' % base)
             try:
-                module = __import__(path)
+                __import__(path)
                 todoc = getattr(sys.modules[path], base)
             except (ImportError, AttributeError):
                 raise InheritanceException(
@@ -130,39 +134,68 @@ class InheritanceGraph(object):
                                    'not a class or module' % name)
 
     def _import_classes(self, class_names, currmodule):
-        """
-        Import a list of classes.
-        """
+        """Import a list of classes."""
         classes = []
         for name in class_names:
             classes.extend(self._import_class_or_module(name, currmodule))
         return classes
 
-    def _all_classes(self, classes):
-        """
-        Return a list of all classes that are ancestors of *classes*.
+    def _class_info(self, classes, show_builtins, private_bases, parts):
+        """Return name and bases for all classes that are ancestors of
+        *classes*.
+
+        *parts* gives the number of dotted name parts that is removed from the
+        displayed node names.
         """
         all_classes = {}
+        py_builtins = vars(builtins).values()
 
         def recurse(cls):
-            all_classes[cls] = None
-            for c in cls.__bases__:
-                if c not in all_classes:
-                    recurse(c)
+            if not show_builtins and cls in py_builtins:
+                return
+            if not private_bases and cls.__name__.startswith('_'):
+                return
+
+            nodename = self.class_name(cls, parts)
+            fullname = self.class_name(cls, 0)
+
+            # Use first line of docstring as tooltip, if available
+            tooltip = None
+            try:
+                if cls.__doc__:
+                    enc = ModuleAnalyzer.for_module(cls.__module__).encoding
+                    doc = cls.__doc__.strip().split("\n")[0]
+                    if not isinstance(doc, text_type):
+                        doc = force_decode(doc, enc)
+                    if doc:
+                        tooltip = '"%s"' % doc.replace('"', '\\"')
+            except Exception:  # might raise AttributeError for strange classes
+                pass
+
+            baselist = []
+            all_classes[cls] = (nodename, fullname, baselist, tooltip)
+            for base in cls.__bases__:
+                if not show_builtins and base in py_builtins:
+                    continue
+                if not private_bases and base.__name__.startswith('_'):
+                    continue
+                baselist.append(self.class_name(base, parts))
+                if base not in all_classes:
+                    recurse(base)
 
         for cls in classes:
             recurse(cls)
 
-        return all_classes.keys()
+        return list(all_classes.values())
 
     def class_name(self, cls, parts=0):
-        """
-        Given a class object, return a fully-qualified name.  This
-        works for things I've tested in matplotlib so far, but may not
-        be completely general.
+        """Given a class object, return a fully-qualified name.
+
+        This works for things I've tested in matplotlib so far, but may not be
+        completely general.
         """
         module = cls.__module__
-        if module == '__builtin__':
+        if module in ('__builtin__', 'builtins'):
             fullname = cls.__name__
         else:
             fullname = '%s.%s' % (module, cls.__name__)
@@ -172,10 +205,8 @@ class InheritanceGraph(object):
         return '.'.join(name_parts[-parts:])
 
     def get_all_class_names(self):
-        """
-        Get all of the class names involved in the graph.
-        """
-        return [self.class_name(x) for x in self.all_classes]
+        """Get all of the class names involved in the graph."""
+        return [fullname for (_, fullname, _, _) in self.class_info]
 
     # These are the default attrs for graphviz
     default_graph_attrs = {
@@ -186,8 +217,8 @@ class InheritanceGraph(object):
         'shape': 'box',
         'fontsize': 10,
         'height': 0.25,
-        'fontname': 'Vera Sans, DejaVu Sans, Liberation Sans, '
-                    'Arial, Helvetica, sans',
+        'fontname': '"Vera Sans, DejaVu Sans, Liberation Sans, '
+                    'Arial, Helvetica, sans"',
         'style': '"setlinewidth(0.5)"',
     }
     default_edge_attrs = {
@@ -196,16 +227,15 @@ class InheritanceGraph(object):
     }
 
     def _format_node_attrs(self, attrs):
-        return ','.join(['%s=%s' % x for x in attrs.items()])
+        return ','.join(['%s=%s' % x for x in sorted(attrs.items())])
 
     def _format_graph_attrs(self, attrs):
-        return ''.join(['%s=%s;\n' % x for x in attrs.items()])
+        return ''.join(['%s=%s;\n' % x for x in sorted(attrs.items())])
 
-    def generate_dot(self, name, parts=0, urls={}, env=None,
+    def generate_dot(self, name, urls={}, env=None,
                      graph_attrs={}, node_attrs={}, edge_attrs={}):
-        """
-        Generate a graphviz dot graph from the classes that
-        were passed in to __init__.
+        """Generate a graphviz dot graph from the classes that were passed in
+        to __init__.
 
         *name* is the name of the graph.
 
@@ -229,26 +259,19 @@ class InheritanceGraph(object):
         res.append('digraph %s {\n' % name)
         res.append(self._format_graph_attrs(g_attrs))
 
-        for cls in self.all_classes:
-            if not self.show_builtins and cls in __builtins__.values():
-                continue
-
-            name = self.class_name(cls, parts)
-
+        for name, fullname, bases, tooltip in sorted(self.class_info):
             # Write the node
             this_node_attrs = n_attrs.copy()
-            url = urls.get(self.class_name(cls))
-            if url is not None:
-                this_node_attrs['URL'] = '"%s"' % url
+            if fullname in urls:
+                this_node_attrs['URL'] = '"%s"' % urls[fullname]
+                this_node_attrs['target'] = '"_top"'
+            if tooltip:
+                this_node_attrs['tooltip'] = tooltip
             res.append('  "%s" [%s];\n' %
                        (name, self._format_node_attrs(this_node_attrs)))
 
             # Write the edges
-            for base in cls.__bases__:
-                if not self.show_builtins and base in __builtins__.values():
-                    continue
-
-                base_name = self.class_name(base, parts)
+            for base_name in bases:
                 res.append('  "%s" -> "%s" [%s];\n' %
                            (base_name, name,
                             self._format_node_attrs(e_attrs)))
@@ -273,6 +296,7 @@ class InheritanceDiagram(Directive):
     final_argument_whitespace = True
     option_spec = {
         'parts': directives.nonnegative_int,
+        'private-bases': directives.flag,
     }
 
     def run(self):
@@ -280,11 +304,18 @@ class InheritanceDiagram(Directive):
         node.document = self.state.document
         env = self.state.document.settings.env
         class_names = self.arguments[0].split()
+        class_role = env.get_domain('py').role('class')
+        # Store the original content for use as a hash
+        node['parts'] = self.options.get('parts', 0)
+        node['content'] = ', '.join(class_names)
 
         # Create a graph starting with the list of classes
         try:
-            graph = InheritanceGraph(class_names, env.currmodule)
-        except InheritanceException, err:
+            graph = InheritanceGraph(
+                class_names, env.ref_context.get('py:module'),
+                parts=node['parts'],
+                private_bases='private-bases' in self.options)
+        except InheritanceException as err:
             return [node.document.reporter.warning(err.args[0],
                                                    line=self.lineno)]
 
@@ -293,20 +324,18 @@ class InheritanceDiagram(Directive):
         # references to real URLs later.  These nodes will eventually be
         # removed from the doctree after we're done with them.
         for name in graph.get_all_class_names():
-            refnodes, x = xfileref_role(
+            refnodes, x = class_role(
                 'class', ':class:`%s`' % name, name, 0, self.state)
             node.extend(refnodes)
         # Store the graph object so we can use it to generate the
         # dot file later
         node['graph'] = graph
-        # Store the original content for use as a hash
-        node['parts'] = self.options.get('parts', 0)
-        node['content'] = ', '.join(class_names)
         return [node]
 
 
 def get_graph_hash(node):
-    return md5(node['content'] + str(node['parts'])).hexdigest()[-10:]
+    encoded = (node['content'] + str(node['parts'])).encode('utf-8')
+    return md5(encoded).hexdigest()[-10:]
 
 
 def html_visit_inheritance_diagram(self, node):
@@ -315,20 +344,27 @@ def html_visit_inheritance_diagram(self, node):
     image map.
     """
     graph = node['graph']
-    parts = node['parts']
 
     graph_hash = get_graph_hash(node)
     name = 'inheritance%s' % graph_hash
 
     # Create a mapping from fully-qualified class names to URLs.
+    graphviz_output_format = self.builder.env.config.graphviz_output_format.upper()
+    current_filename = self.builder.current_docname + self.builder.out_suffix
     urls = {}
     for child in node:
         if child.get('refuri') is not None:
-            urls[child['reftitle']] = child.get('refuri')
+            if graphviz_output_format == 'SVG':
+                urls[child['reftitle']] = "../" + child.get('refuri')
+            else:
+                urls[child['reftitle']] = child.get('refuri')
         elif child.get('refid') is not None:
-            urls[child['reftitle']] = '#' + child.get('refid')
+            if graphviz_output_format == 'SVG':
+                urls[child['reftitle']] = '../' + current_filename + '#' + child.get('refid')
+            else:
+                urls[child['reftitle']] = '#' + child.get('refid')
 
-    dotcode = graph.generate_dot(name, parts, urls, env=self.builder.env)
+    dotcode = graph.generate_dot(name, urls, env=self.builder.env)
     render_dot_html(self, node, dotcode, [], 'inheritance', 'inheritance',
                     alt='Inheritance diagram of ' + node['content'])
     raise nodes.SkipNode
@@ -339,17 +375,31 @@ def latex_visit_inheritance_diagram(self, node):
     Output the graph for LaTeX.  This will insert a PDF.
     """
     graph = node['graph']
-    parts = node['parts']
 
     graph_hash = get_graph_hash(node)
     name = 'inheritance%s' % graph_hash
 
-    dotcode = graph.generate_dot(name, parts, env=self.builder.env,
+    dotcode = graph.generate_dot(name, env=self.builder.env,
                                  graph_attrs={'size': '"6.0,6.0"'})
     render_dot_latex(self, node, dotcode, [], 'inheritance')
     raise nodes.SkipNode
 
 
+def texinfo_visit_inheritance_diagram(self, node):
+    """
+    Output the graph for Texinfo.  This will insert a PNG.
+    """
+    graph = node['graph']
+
+    graph_hash = get_graph_hash(node)
+    name = 'inheritance%s' % graph_hash
+
+    dotcode = graph.generate_dot(name, env=self.builder.env,
+                                 graph_attrs={'size': '"6.0,6.0"'})
+    render_dot_texinfo(self, node, dotcode, [], 'inheritance')
+    raise nodes.SkipNode
+
+
 def skip(self, node):
     raise nodes.SkipNode
 
@@ -360,8 +410,11 @@ def setup(app):
         inheritance_diagram,
         latex=(latex_visit_inheritance_diagram, None),
         html=(html_visit_inheritance_diagram, None),
-        text=(skip, None))
+        text=(skip, None),
+        man=(skip, None),
+        texinfo=(texinfo_visit_inheritance_diagram, None))
     app.add_directive('inheritance-diagram', InheritanceDiagram)
     app.add_config_value('inheritance_graph_attrs', {}, False),
     app.add_config_value('inheritance_node_attrs', {}, False),
     app.add_config_value('inheritance_edge_attrs', {}, False),
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
