@@ -5,35 +5,44 @@
 
     Render math in HTML via dvipng.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
+import codecs
 import shutil
 import tempfile
 import posixpath
-from os import path, getcwd, chdir
+from os import path
 from subprocess import Popen, PIPE
-try:
-    from hashlib import sha1 as sha
-except ImportError:
-    from sha import sha
+from hashlib import sha1
 
+from six import text_type
 from docutils import nodes
 
+import sphinx
 from sphinx.errors import SphinxError
-from sphinx.util import ensuredir, ENOENT
 from sphinx.util.png import read_png_depth, write_png_depth
+from sphinx.util.osutil import ensuredir, ENOENT, cd
+from sphinx.util.pycompat import sys_encoding
 from sphinx.ext.mathbase import setup_math as mathbase_setup, wrap_displaymath
 
+
 class MathExtError(SphinxError):
     category = 'Math extension error'
 
+    def __init__(self, msg, stderr=None, stdout=None):
+        if stderr:
+            msg += '\n[stderr]\n' + stderr.decode(sys_encoding, 'replace')
+        if stdout:
+            msg += '\n[stdout]\n' + stdout.decode(sys_encoding, 'replace')
+        SphinxError.__init__(self, msg)
+
 
 DOC_HEAD = r'''
 \documentclass[12pt]{article}
-\usepackage[utf8]{inputenc}
+\usepackage[utf8x]{inputenc}
 \usepackage{amsmath}
 \usepackage{amsthm}
 \usepackage{amssymb}
@@ -57,11 +66,11 @@ DOC_BODY_PREVIEW = r'''
 \end{document}
 '''
 
-depth_re = re.compile(r'\[\d+ depth=(-?\d+)\]')
+depth_re = re.compile(br'\[\d+ depth=(-?\d+)\]')
+
 
 def render_math(self, math):
-    """
-    Render the LaTeX math expression *math* using latex and dvipng.
+    """Render the LaTeX math expression *math* using latex and dvipng.
 
     Return the filename relative to the built document and the "depth",
     that is, the distance of image bottom and baseline in pixels, if the
@@ -74,10 +83,12 @@ def render_math(self, math):
     may not fail since that indicates a problem in the math source.
     """
     use_preview = self.builder.config.pngmath_use_preview
+    latex = DOC_HEAD + self.builder.config.pngmath_latex_preamble
+    latex += (use_preview and DOC_BODY_PREVIEW or DOC_BODY) % math
 
-    shasum = "%s.png" % sha(math.encode('utf-8')).hexdigest()
+    shasum = "%s.png" % sha1(latex.encode('utf-8')).hexdigest()
     relfn = posixpath.join(self.builder.imgpath, 'math', shasum)
-    outfn = path.join(self.builder.outdir, '_images', 'math', shasum)
+    outfn = path.join(self.builder.outdir, self.builder.imagedir, 'math', shasum)
     if path.isfile(outfn):
         depth = read_png_depth(outfn)
         return relfn, depth
@@ -87,11 +98,6 @@ def render_math(self, math):
        hasattr(self.builder, '_mathpng_warned_dvipng'):
         return None, None
 
-    latex = DOC_HEAD + self.builder.config.pngmath_latex_preamble
-    latex += (use_preview and DOC_BODY_PREVIEW or DOC_BODY) % math
-    if isinstance(latex, unicode):
-        latex = latex.encode('utf-8')
-
     # use only one tempdir per build -- the use of a directory is cleaner
     # than using temporary files, since we can clean up everything at once
     # just removing the whole directory (see cleanup_tempdir)
@@ -100,7 +106,7 @@ def render_math(self, math):
     else:
         tempdir = self.builder._mathpng_tempdir
 
-    tf = open(path.join(tempdir, 'math.tex'), 'w')
+    tf = codecs.open(path.join(tempdir, 'math.tex'), 'w', 'utf-8')
     tf.write(latex)
     tf.close()
 
@@ -112,13 +118,10 @@ def render_math(self, math):
     ltx_args.extend(self.builder.config.pngmath_latex_args)
     ltx_args.append('math.tex')
 
-    curdir = getcwd()
-    chdir(tempdir)
-
-    try:
+    with cd(tempdir):
         try:
             p = Popen(ltx_args, stdout=PIPE, stderr=PIPE)
-        except OSError, err:
+        except OSError as err:
             if err.errno != ENOENT:   # No such file or directory
                 raise
             self.builder.warn('LaTeX command %r cannot be run (needed for math '
@@ -126,13 +129,10 @@ def render_math(self, math):
                               self.builder.config.pngmath_latex)
             self.builder._mathpng_warned_latex = True
             return None, None
-    finally:
-        chdir(curdir)
 
     stdout, stderr = p.communicate()
     if p.returncode != 0:
-        raise MathExtError('latex exited with error:\n[stderr]\n%s\n'
-                           '[stdout]\n%s' % (stderr, stdout))
+        raise MathExtError('latex exited with error', stderr, stdout)
 
     ensuredir(path.dirname(outfn))
     # use some standard dvipng arguments
@@ -146,7 +146,7 @@ def render_math(self, math):
     dvipng_args.append(path.join(tempdir, 'math.dvi'))
     try:
         p = Popen(dvipng_args, stdout=PIPE, stderr=PIPE)
-    except OSError, err:
+    except OSError as err:
         if err.errno != ENOENT:   # No such file or directory
             raise
         self.builder.warn('dvipng command %r cannot be run (needed for math '
@@ -156,8 +156,7 @@ def render_math(self, math):
         return None, None
     stdout, stderr = p.communicate()
     if p.returncode != 0:
-        raise MathExtError('dvipng exited with error:\n[stderr]\n%s\n'
-                           '[stdout]\n%s' % (stderr, stdout))
+        raise MathExtError('dvipng exited with error', stderr, stdout)
     depth = None
     if use_preview:
         for line in stdout.splitlines():
@@ -169,6 +168,7 @@ def render_math(self, math):
 
     return relfn, depth
 
+
 def cleanup_tempdir(app, exc):
     if exc:
         return
@@ -179,31 +179,35 @@ def cleanup_tempdir(app, exc):
     except Exception:
         pass
 
+
+def get_tooltip(self, node):
+    if self.builder.config.pngmath_add_tooltips:
+        return ' alt="%s"' % self.encode(node['latex']).strip()
+    return ''
+
+
 def html_visit_math(self, node):
     try:
         fname, depth = render_math(self, '$'+node['latex']+'$')
-    except MathExtError, exc:
-        sm = nodes.system_message(str(exc), type='WARNING', level=2,
+    except MathExtError as exc:
+        msg = text_type(exc)
+        sm = nodes.system_message(msg, type='WARNING', level=2,
                                   backrefs=[], source=node['latex'])
         sm.walkabout(self)
-        self.builder.warn('display latex %r: ' % node['latex'] + str(exc))
+        self.builder.warn('display latex %r: ' % node['latex'] + msg)
         raise nodes.SkipNode
     if fname is None:
         # something failed -- use text-only as a bad substitute
         self.body.append('<span class="math">%s</span>' %
                          self.encode(node['latex']).strip())
     else:
-        if depth is None:
-            self.body.append(
-                '<img class="math" src="%s" alt="%s"/>' %
-                (fname, self.encode(node['latex']).strip()))
-        else:
-            self.body.append(
-                '<img class="math" src="%s" alt="%s" '
-                'style="vertical-align: %dpx"/>' %
-                (fname, self.encode(node['latex']).strip(), -depth))
+        c = ('<img class="math" src="%s"' % fname) + get_tooltip(self, node)
+        if depth is not None:
+            c += ' style="vertical-align: %dpx"' % (-depth)
+        self.body.append(c + '/>')
     raise nodes.SkipNode
 
+
 def html_visit_displaymath(self, node):
     if node['nowrap']:
         latex = node['latex']
@@ -211,7 +215,7 @@ def html_visit_displaymath(self, node):
         latex = wrap_displaymath(node['latex'], None)
     try:
         fname, depth = render_math(self, latex)
-    except MathExtError, exc:
+    except MathExtError as exc:
         sm = nodes.system_message(str(exc), type='WARNING', level=2,
                                   backrefs=[], source=node['latex'])
         sm.walkabout(self)
@@ -226,8 +230,8 @@ def html_visit_displaymath(self, node):
         self.body.append('<span class="math">%s</span></p>\n</div>' %
                          self.encode(node['latex']).strip())
     else:
-        self.body.append('<img src="%s" alt="%s" /></p>\n</div>' %
-                         (fname, self.encode(node['latex']).strip()))
+        self.body.append(('<img src="%s"' % fname) + get_tooltip(self, node) +
+                         '/></p>\n</div>')
     raise nodes.SkipNode
 
 
@@ -237,7 +241,10 @@ def setup(app):
     app.add_config_value('pngmath_latex', 'latex', 'html')
     app.add_config_value('pngmath_use_preview', False, 'html')
     app.add_config_value('pngmath_dvipng_args',
-                         ['-gamma 1.5', '-D 110'], 'html')
+                         ['-gamma', '1.5', '-D', '110', '-bg', 'Transparent'],
+                         'html')
     app.add_config_value('pngmath_latex_args', [], 'html')
     app.add_config_value('pngmath_latex_preamble', '', 'html')
+    app.add_config_value('pngmath_add_tooltips', True, 'html')
     app.connect('build-finished', cleanup_tempdir)
+    return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
