@@ -5,7 +5,7 @@
 
     Format colored console output.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -13,13 +13,22 @@ import os
 import sys
 import re
 
+try:
+    # check if colorama is installed to support color on Windows
+    import colorama
+except ImportError:
+    colorama = None
+
 _ansi_re = re.compile('\x1b\\[(\\d\\d;){0,2}\\d\\dm')
 codes = {}
 
+
 def get_terminal_width():
     """Borrowed from the py lib."""
     try:
-        import termios, fcntl, struct
+        import termios
+        import fcntl
+        import struct
         call = fcntl.ioctl(0, termios.TIOCGWINSZ,
                            struct.pack('hhhh', 0, 0, 0, 0))
         height, width = struct.unpack('hhhh', call)[:2]
@@ -33,6 +42,8 @@ def get_terminal_width():
 
 
 _tw = get_terminal_width()
+
+
 def term_width_line(text):
     if not codes:
         # if no coloring, don't output fancy backspaces
@@ -41,7 +52,11 @@ def term_width_line(text):
         # codes are not displayed, this must be taken into account
         return text.ljust(_tw + len(text) - len(_ansi_re.sub('', text))) + '\r'
 
+
 def color_terminal():
+    if sys.platform == 'win32' and colorama is not None:
+        colorama.init()
+        return True
     if not hasattr(sys.stdout, 'isatty'):
         return False
     if not sys.stdout.isatty():
@@ -55,14 +70,23 @@ def color_terminal():
 
 
 def nocolor():
+    if sys.platform == 'win32' and colorama is not None:
+        colorama.deinit()
     codes.clear()
 
+
 def coloron():
     codes.update(_orig_codes)
 
+
 def colorize(name, text):
     return codes.get(name, '') + text + codes.get('reset', '')
 
+
+def strip_colors(s):
+    return re.compile('\x1b.*?m').sub('', s)
+
+
 def create_color_func(name):
     def inner(text):
         return colorize(name, text)
