@@ -6,12 +6,16 @@
     This module implements a simple JavaScript serializer.
     Uses the basestring encode function from simplejson by Bob Ippolito.
 
-    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import re
 
+from six import iteritems, integer_types, string_types
+
+from sphinx.util.pycompat import u
+
 _str_re  = re.compile(r'"(\\\\|\\"|[^"])*"')
 _int_re  = re.compile(r'\d+')
 _name_re = re.compile(r'[a-zA-Z]\w*')
@@ -49,8 +53,9 @@ def encode_string(s):
                 return '\\u%04x\\u%04x' % (s1, s2)
     return '"' + str(ESCAPE_ASCII.sub(replace, s)) + '"'
 
+
 def decode_string(s):
-    return ESCAPED.sub(lambda m: eval('u"'+m.group()+'"'), s)
+    return ESCAPED.sub(lambda m: eval(u + '"' + m.group() + '"'), s)
 
 
 reswords = set("""\
@@ -70,9 +75,10 @@ delete   implements   short   while
 do   import   static   with
 double   in   super""".split())
 
+
 def dumps(obj, key=False):
     if key:
-        if not isinstance(obj, basestring):
+        if not isinstance(obj, string_types):
             obj = str(obj)
         if _nameonly_re.match(obj) and obj not in reswords:
             return obj  # return it as a bare word
@@ -82,19 +88,22 @@ def dumps(obj, key=False):
         return 'null'
     elif obj is True or obj is False:
         return obj and 'true' or 'false'
-    elif isinstance(obj, (int, long, float)):
+    elif isinstance(obj, integer_types + (float,)):
         return str(obj)
     elif isinstance(obj, dict):
-        return '{%s}' % ','.join('%s:%s' % (
+        return '{%s}' % ','.join(sorted('%s:%s' % (
             dumps(key, True),
             dumps(value)
-        ) for key, value in obj.iteritems())
-    elif isinstance(obj, (tuple, list, set)):
+        ) for key, value in iteritems(obj)))
+    elif isinstance(obj, set):
+        return '[%s]' % ','.join(sorted(dumps(x) for x in obj))
+    elif isinstance(obj, (tuple, list)):
         return '[%s]' % ','.join(dumps(x) for x in obj)
-    elif isinstance(obj, basestring):
+    elif isinstance(obj, string_types):
         return encode_string(obj)
     raise TypeError(type(obj))
 
+
 def dump(obj, f):
     f.write(dumps(obj))
 
@@ -189,5 +198,6 @@ def loads(x):
         raise ValueError("nothing loaded from string")
     return obj
 
+
 def load(f):
     return loads(f.read())
