@@ -6,24 +6,24 @@
     Test the autodoc extension.  This tests mainly the Documenters; the auto
     directives are tested in a test source file translated by test_build.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import sys
-from StringIO import StringIO
-
-from util import *
+# "raises" imported for usage by autodoc
+from util import TestApp, Struct, raises
+from nose.tools import with_setup
 
+from six import StringIO
 from docutils.statemachine import ViewList
 
 from sphinx.ext.autodoc import AutoDirective, add_documenter, \
-     ModuleLevelDocumenter, FunctionDocumenter, cut_lines, between, ALL
+    ModuleLevelDocumenter, FunctionDocumenter, cut_lines, between, ALL
 
+app = None
 
 def setup_module():
-    global app, lid, options, directive
-
+    global app
     app = TestApp()
     app.builder.env.app = app
     app.builder.env.temp_data['docname'] = 'dummy'
@@ -31,13 +31,26 @@ def setup_module():
     app.connect('autodoc-process-signature', process_signature)
     app.connect('autodoc-skip-member', skip_member)
 
+
+def teardown_module():
+    app.cleanup()
+
+
+directive = options = None
+
+def setup_test():
+    global options, directive
+    global processed_docstrings, processed_signatures, _warnings
+
     options = Struct(
         inherited_members = False,
         undoc_members = False,
         private_members = False,
         special_members = False,
+        imported_members = False,
         show_inheritance = False,
         noindex = False,
+        annotation = None,
         synopsis = '',
         platform = '',
         deprecated = False,
@@ -54,8 +67,9 @@ def setup_module():
         filename_set = set(),
     )
 
-def teardown_module():
-    app.cleanup()
+    processed_docstrings = []
+    processed_signatures = []
+    _warnings = []
 
 
 _warnings = []
@@ -80,12 +94,15 @@ def process_signature(app, what, name, obj, options, args, retann):
 
 
 def skip_member(app, what, name, obj, skip, options):
+    if name in ('__special1__', '__special2__'):
+        return skip
     if name.startswith('_'):
         return True
     if name == 'skipmeth':
         return True
 
 
+@with_setup(setup_test)
 def test_parse_name():
     def verify(objtype, name, result):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -106,36 +123,40 @@ def test_parse_name():
     directive.env.temp_data['autodoc:module'] = 'util'
     verify('function', 'raises', ('util', ['raises'], None, None))
     del directive.env.temp_data['autodoc:module']
-    directive.env.temp_data['py:module'] = 'util'
+    directive.env.ref_context['py:module'] = 'util'
     verify('function', 'raises', ('util', ['raises'], None, None))
     verify('class', 'TestApp', ('util', ['TestApp'], None, None))
 
     # for members
-    directive.env.temp_data['py:module'] = 'foo'
+    directive.env.ref_context['py:module'] = 'foo'
     verify('method', 'util.TestApp.cleanup',
            ('util', ['TestApp', 'cleanup'], None, None))
-    directive.env.temp_data['py:module'] = 'util'
-    directive.env.temp_data['py:class'] = 'Foo'
+    directive.env.ref_context['py:module'] = 'util'
+    directive.env.ref_context['py:class'] = 'Foo'
     directive.env.temp_data['autodoc:class'] = 'TestApp'
     verify('method', 'cleanup', ('util', ['TestApp', 'cleanup'], None, None))
     verify('method', 'TestApp.cleanup',
            ('util', ['TestApp', 'cleanup'], None, None))
 
     # and clean up
-    del directive.env.temp_data['py:module']
-    del directive.env.temp_data['py:class']
+    del directive.env.ref_context['py:module']
+    del directive.env.ref_context['py:class']
     del directive.env.temp_data['autodoc:class']
 
 
+@with_setup(setup_test)
 def test_format_signature():
     def formatsig(objtype, name, obj, args, retann):
         inst = AutoDirective._registry[objtype](directive, name)
         inst.fullname = name
         inst.doc_as_attr = False  # for class objtype
         inst.object = obj
+        inst.objpath = [name]
         inst.args = args
         inst.retann = retann
-        return inst.format_signature()
+        res = inst.format_signature()
+        print(res)
+        return res
 
     # no signatures for modules
     assert formatsig('module', 'test', None, None, None) == ''
@@ -143,10 +164,13 @@ def test_format_signature():
     # test for functions
     def f(a, b, c=1, **d):
         pass
+    def g(a='\n'):
+        pass
     assert formatsig('function', 'f', f, None, None) == '(a, b, c=1, **d)'
     assert formatsig('function', 'f', f, 'a, b, c, d', None) == '(a, b, c, d)'
     assert formatsig('function', 'f', f, None, 'None') == \
            '(a, b, c=1, **d) -> None'
+    assert formatsig('function', 'g', g, None, None) == r"(a='\\n')"
 
     # test for classes
     class D:
@@ -165,31 +189,74 @@ def test_format_signature():
         assert formatsig('class', 'C', C, None, None) == '(a, b=None)'
     assert formatsig('class', 'C', D, 'a, b', 'X') == '(a, b) -> X'
 
+    # __init__ have signature at first line of docstring
+    directive.env.config.autoclass_content = 'both'
+    class F2:
+        '''some docstring for F2.'''
+        def __init__(self, *args, **kw):
+            '''
+            __init__(a1, a2, kw1=True, kw2=False)
+
+            some docstring for __init__.
+            '''
+    class G2(F2, object):
+        pass
+
+    assert formatsig('class', 'F2', F2, None, None) == \
+        '(a1, a2, kw1=True, kw2=False)'
+    assert formatsig('class', 'G2', G2, None, None) == \
+        '(a1, a2, kw1=True, kw2=False)'
+
     # test for methods
     class H:
         def foo1(self, b, *c):
             pass
         def foo2(b, *c):
             pass
+        def foo3(self, d='\n'):
+            pass
     assert formatsig('method', 'H.foo', H.foo1, None, None) == '(b, *c)'
     assert formatsig('method', 'H.foo', H.foo1, 'a', None) == '(a)'
     assert formatsig('method', 'H.foo', H.foo2, None, None) == '(b, *c)'
+    assert formatsig('method', 'H.foo', H.foo3, None, None) == r"(d='\\n')"
 
     # test exception handling (exception is caught and args is '')
+    directive.env.config.autodoc_docstring_signature = False
     assert formatsig('function', 'int', int, None, None) == ''
     del _warnings[:]
 
     # test processing by event handler
     assert formatsig('method', 'bar', H.foo1, None, None) == '42'
 
+    # test functions created via functools.partial
+    from functools import partial
+    curried1 = partial(lambda a, b, c: None, 'A')
+    assert formatsig('function', 'curried1', curried1, None, None) == \
+        '(b, c)'
+    curried2 = partial(lambda a, b, c=42: None, 'A')
+    assert formatsig('function', 'curried2', curried2, None, None) == \
+        '(b, c=42)'
+    curried3 = partial(lambda a, b, *c: None, 'A')
+    assert formatsig('function', 'curried3', curried3, None, None) == \
+        '(b, *c)'
+    curried4 = partial(lambda a, b, c=42, *d, **e: None, 'A')
+    assert formatsig('function', 'curried4', curried4, None, None) == \
+        '(b, c=42, *d, **e)'
 
+
+@with_setup(setup_test)
 def test_get_doc():
     def getdocl(objtype, obj, encoding=None):
         inst = AutoDirective._registry[objtype](directive, 'tmp')
         inst.object = obj
+        inst.objpath = [obj.__name__]
+        inst.doc_as_attr = False
+        inst.format_signature()  # handle docstring signatures!
         ds = inst.get_doc(encoding)
         # for testing purposes, concat them and strip the empty line at the end
-        return sum(ds, [])[:-1]
+        res = sum(ds, [])[:-1]
+        print(res)
+        return res
 
     # objects without docstring
     def f():
@@ -250,7 +317,71 @@ def test_get_doc():
     assert getdocl('class', D) == ['Class docstring', '', 'Init docstring',
                                    '', 'Other', ' lines']
 
+    # __init__ have signature at first line of docstring
+    class E:
+        """Class docstring"""
+        def __init__(self, *args, **kw):
+            """
+            __init__(a1, a2, kw1=True, kw2=False)
 
+            Init docstring
+            """
+
+    # signature line in the docstring will be kept when
+    # autodoc_docstring_signature == False
+    directive.env.config.autodoc_docstring_signature = False
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', E) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', E) == ['__init__(a1, a2, kw1=True, kw2=False)',
+                                   '', 'Init docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', E) == ['Class docstring', '',
+                                   '__init__(a1, a2, kw1=True, kw2=False)',
+                                   '', 'Init docstring']
+
+    # signature line in the docstring will be removed when
+    # autodoc_docstring_signature == True
+    directive.env.config.autodoc_docstring_signature = True  # default
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', E) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', E) == ['Init docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', E) == ['Class docstring', '', 'Init docstring']
+
+    # class does not have __init__ method
+    class F(object):
+        """Class docstring"""
+
+    # docstring in the __init__ method of base class will be discard
+    for f in (False, True):
+        directive.env.config.autodoc_docstring_signature = f
+        directive.env.config.autoclass_content = 'class'
+        assert getdocl('class', F) == ['Class docstring']
+        directive.env.config.autoclass_content = 'init'
+        assert getdocl('class', F) == ['Class docstring']
+        directive.env.config.autoclass_content = 'both'
+        assert getdocl('class', F) == ['Class docstring']
+
+    # class has __init__ method with no docstring
+    class G(object):
+        """Class docstring"""
+        def __init__(self):
+            pass
+
+    # docstring in the __init__ method of base class will not be used
+    for f in (False, True):
+        directive.env.config.autodoc_docstring_signature = f
+        directive.env.config.autoclass_content = 'class'
+        assert getdocl('class', G) == ['Class docstring']
+        directive.env.config.autoclass_content = 'init'
+        assert getdocl('class', G) == ['Class docstring']
+        directive.env.config.autoclass_content = 'both'
+        assert getdocl('class', G) == ['Class docstring']
+
+
+@with_setup(setup_test)
 def test_docstring_processing():
     def process(objtype, name, obj):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -301,6 +432,45 @@ def test_docstring_processing():
     assert process('function', 'h', h) == ['first line', 'third line', '']
     app.disconnect(lid)
 
+
+@with_setup(setup_test)
+def test_docstring_property_processing():
+    def genarate_docstring(objtype, name, **kw):
+        del processed_docstrings[:]
+        del processed_signatures[:]
+        inst = AutoDirective._registry[objtype](directive, name)
+        inst.generate(**kw)
+        results = list(directive.result)
+        docstrings = inst.get_doc()[0]
+        del directive.result[:]
+        return results, docstrings
+
+    directive.env.config.autodoc_docstring_signature = False
+    results, docstrings = \
+        genarate_docstring('attribute', 'test_autodoc.DocstringSig.prop1')
+    assert '.. py:attribute:: DocstringSig.prop1' in results
+    assert 'First line of docstring' in docstrings
+    assert 'DocstringSig.prop1(self)' in docstrings
+    results, docstrings = \
+        genarate_docstring('attribute', 'test_autodoc.DocstringSig.prop2')
+    assert '.. py:attribute:: DocstringSig.prop2' in results
+    assert 'First line of docstring' in docstrings
+    assert 'Second line of docstring' in docstrings
+
+    directive.env.config.autodoc_docstring_signature = True
+    results, docstrings = \
+        genarate_docstring('attribute', 'test_autodoc.DocstringSig.prop1')
+    assert '.. py:attribute:: DocstringSig.prop1' in results
+    assert 'First line of docstring' in docstrings
+    assert 'DocstringSig.prop1(self)' not in docstrings
+    results, docstrings = \
+        genarate_docstring('attribute', 'test_autodoc.DocstringSig.prop2')
+    assert '.. py:attribute:: DocstringSig.prop2' in results
+    assert 'First line of docstring' in docstrings
+    assert 'Second line of docstring' in docstrings
+
+
+@with_setup(setup_test)
 def test_new_documenter():
     class MyDocumenter(ModuleLevelDocumenter):
         objtype = 'integer'
@@ -328,6 +498,40 @@ def test_new_documenter():
     assert_result_contains('.. py:data:: integer', 'module', 'test_autodoc')
 
 
+@with_setup(setup_test, AutoDirective._special_attrgetters.clear)
+def test_attrgetter_using():
+    def assert_getter_works(objtype, name, obj, attrs=[], **kw):
+        getattr_spy = []
+        def special_getattr(obj, name, *defargs):
+            if name in attrs:
+                getattr_spy.append((obj, name))
+                return None
+            return getattr(obj, name, *defargs)
+        AutoDirective._special_attrgetters[type] = special_getattr
+
+        del getattr_spy[:]
+        inst = AutoDirective._registry[objtype](directive, name)
+        inst.generate(**kw)
+
+        hooked_members = [s[1] for s in getattr_spy]
+        documented_members = [s[1] for s in processed_signatures]
+        for attr in attrs:
+            fullname = '.'.join((name, attr))
+            assert attr in hooked_members
+            assert fullname not in documented_members, \
+                '%r was not hooked by special_attrgetter function' % fullname
+
+    options.members = ALL
+    options.inherited_members = False
+    assert_getter_works('class', 'test_autodoc.Class', Class,
+                       ['meth'])
+
+    options.inherited_members = True
+    assert_getter_works('class', 'test_autodoc.Class', Class,
+                       ['meth', 'inheritedmeth'])
+
+
+@with_setup(setup_test)
 def test_generate():
     def assert_warns(warn_str, objtype, name, **kw):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -386,14 +590,17 @@ def test_generate():
     assert_warns("import for autodocumenting 'foobar'",
                  'function', 'foobar', more_content=None)
     # importing
-    assert_warns("import/find module 'test_foobar'",
+    assert_warns("failed to import module 'test_foobar'",
                  'module', 'test_foobar', more_content=None)
     # attributes missing
-    assert_warns("import/find function 'util.foobar'",
+    assert_warns("failed to import function 'foobar' from module 'util'",
                  'function', 'util.foobar', more_content=None)
+    # method missing
+    assert_warns("failed to import method 'Class.foobar' from module 'test_autodoc';",
+                 'method', 'test_autodoc.Class.foobar', more_content=None)
 
     # test auto and given content mixing
-    directive.env.temp_data['py:module'] = 'test_autodoc'
+    directive.env.ref_context['py:module'] = 'test_autodoc'
     assert_result_contains('   Function.', 'method', 'Class.meth')
     add_content = ViewList()
     add_content.append('Content.', '', 0)
@@ -440,6 +647,15 @@ def test_generate():
     should.append(('method', 'test_autodoc.Class.inheritedmeth'))
     assert_processes(should, 'class', 'Class')
 
+    # test special members
+    options.special_members = ['__special1__']
+    should.append(('method', 'test_autodoc.Class.__special1__'))
+    assert_processes(should, 'class', 'Class')
+    options.special_members = ALL
+    should.append(('method', 'test_autodoc.Class.__special2__'))
+    assert_processes(should, 'class', 'Class')
+    options.special_members = False
+
     options.members = []
     # test module flags
     assert_result_contains('.. py:module:: test_autodoc',
@@ -482,12 +698,12 @@ def test_generate():
                            'attribute', 'test_autodoc.Class.descr')
 
     # test generation for C modules (which have no source file)
-    directive.env.temp_data['py:module'] = 'time'
+    directive.env.ref_context['py:module'] = 'time'
     assert_processes([('function', 'time.asctime')], 'function', 'asctime')
     assert_processes([('function', 'time.asctime')], 'function', 'asctime')
 
     # test autodoc_member_order == 'source'
-    directive.env.temp_data['py:module'] = 'test_autodoc'
+    directive.env.ref_context['py:module'] = 'test_autodoc'
     assert_order(['.. py:class:: Class(arg)',
                   '   .. py:attribute:: Class.descr',
                   '   .. py:method:: Class.meth()',
@@ -504,7 +720,7 @@ def test_generate():
                   '   .. py:method:: Class.inheritedmeth()',
                   ],
                  'class', 'Class', member_order='bysource', all_members=True)
-    del directive.env.temp_data['py:module']
+    del directive.env.ref_context['py:module']
 
     # test attribute initialized to class instance from other module
     directive.env.temp_data['autodoc:class'] = 'test_autodoc.Class'
@@ -518,12 +734,18 @@ def test_generate():
         'test_autodoc.DocstringSig.meth')
     assert_result_contains(
         '   rest of docstring', 'method', 'test_autodoc.DocstringSig.meth')
+    assert_result_contains(
+        '.. py:method:: DocstringSig.meth2()', 'method',
+        'test_autodoc.DocstringSig.meth2')
+    assert_result_contains(
+        '       indented line', 'method',
+        'test_autodoc.DocstringSig.meth2')
     assert_result_contains(
         '.. py:classmethod:: Class.moore(a, e, f) -> happiness', 'method',
         'test_autodoc.Class.moore')
 
     # test new attribute documenter behavior
-    directive.env.temp_data['py:module'] = 'test_autodoc'
+    directive.env.ref_context['py:module'] = 'test_autodoc'
     options.undoc_members = True
     assert_processes([('class', 'test_autodoc.AttCls'),
                       ('attribute', 'test_autodoc.AttCls.a1'),
@@ -534,6 +756,39 @@ def test_generate():
     assert_result_contains(
         '   :annotation: = None', 'attribute', 'AttCls.a2')
 
+    # test explicit members with instance attributes
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    directive.env.ref_context['py:module'] = 'test_autodoc'
+    options.inherited_members = False
+    options.undoc_members = False
+    options.members = ALL
+    assert_processes([
+        ('class', 'test_autodoc.InstAttCls'),
+        ('attribute', 'test_autodoc.InstAttCls.ca1'),
+        ('attribute', 'test_autodoc.InstAttCls.ca2'),
+        ('attribute', 'test_autodoc.InstAttCls.ca3'),
+        ('attribute', 'test_autodoc.InstAttCls.ia1'),
+        ('attribute', 'test_autodoc.InstAttCls.ia2'),
+    ], 'class', 'InstAttCls')
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    options.members = ['ca1', 'ia1']
+    assert_processes([
+        ('class', 'test_autodoc.InstAttCls'),
+        ('attribute', 'test_autodoc.InstAttCls.ca1'),
+        ('attribute', 'test_autodoc.InstAttCls.ia1'),
+    ], 'class', 'InstAttCls')
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    del directive.env.ref_context['py:module']
+
+    # test descriptor class documentation
+    options.members = ['CustomDataDescriptor']
+    assert_result_contains('.. py:class:: CustomDataDescriptor(doc)',
+                           'module', 'test_autodoc')
+    assert_result_contains('   .. py:method:: CustomDataDescriptor.meth()',
+                           'module', 'test_autodoc')
 
 # --- generate fodder ------------
 
@@ -559,17 +814,17 @@ class CustomDataDescriptor(object):
             return self
         return 42
 
+    def meth(self):
+        """Function."""
+        return "The Answer"
+
 def _funky_classmethod(name, b, c, d, docstring=None):
     """Generates a classmethod for a class from a template by filling out
     some arguments."""
     def template(cls, a, b, c, d=4, e=5, f=6):
         return a, b, c, d, e, f
-    if sys.version_info >= (2, 5):
-        from functools import partial
-        function = partial(template, b=b, c=c, d=d)
-    else:
-        def function(cls, a, e=5, f=6):
-            return template(a, b, c, d, e, f)
+    from functools import partial
+    function = partial(template, b=b, c=c, d=d)
     function.__name__ = name
     function.__doc__ = docstring
     return classmethod(function)
@@ -601,10 +856,9 @@ class Class(Base):
     #: should be documented -- süß
     attr = 'bar'
 
+    @property
     def prop(self):
         """Property."""
-    # stay 2.4 compatible (docstring!)
-    prop = property(prop, doc="Property.")
 
     docattr = 'baz'
     """should likewise be documented -- süß"""
@@ -628,6 +882,13 @@ class Class(Base):
         self.inst_attr_string = None
         """a documented instance attribute"""
 
+    def __special1__(self):
+        """documented special method"""
+
+    def __special2__(self):
+        # undocumented special method
+        pass
+
 
 class CustomDict(dict):
     """Docstring."""
@@ -660,6 +921,27 @@ First line of docstring
         rest of docstring
         """
 
+    def meth2(self):
+        """First line, no signature
+        Second line followed by indentation::
+
+            indented line
+        """
+
+    @property
+    def prop1(self):
+        """DocstringSig.prop1(self)
+        First line of docstring
+        """
+        return 123
+
+    @property
+    def prop2(self):
+        """First line of docstring
+        Second line of docstring
+        """
+        return 456
+
 class StrRepr(str):
     def __repr__(self):
         return self
@@ -667,3 +949,22 @@ class StrRepr(str):
 class AttCls(object):
     a1 = StrRepr('hello\nworld')
     a2 = None
+
+class InstAttCls(object):
+    """Class with documented class and instance attributes."""
+
+    #: Doc comment for class attribute InstAttCls.ca1.
+    #: It can have multiple lines.
+    ca1 = 'a'
+
+    ca2 = 'b'    #: Doc comment for InstAttCls.ca2. One line only.
+
+    ca3 = 'c'
+    """Docstring for class attribute InstAttCls.ca3."""
+
+    def __init__(self):
+        #: Doc comment for instance attribute InstAttCls.ia1
+        self.ia1 = 'd'
+
+        self.ia2 = 'e'
+        """Docstring for instance attribute InstAttCls.ia2."""
