@@ -5,7 +5,7 @@
 
     Test the build process with LaTeX builder with the test root.
 
-    :copyright: Copyright 2007-2015 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2016 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 from __future__ import print_function
@@ -24,18 +24,15 @@ from test_build_html import ENV_WARNINGS
 
 
 LATEX_WARNINGS = ENV_WARNINGS + """\
-None:None: WARNING: unknown option: &option
-None:None: WARNING: citation not found: missing
-None:None: WARNING: no matching candidate for image URI u'foo.\\*'
-WARNING: invalid pair index entry u''
-WARNING: invalid pair index entry u'keyword; '
+%(root)s/footnote.txt:60: WARNING: citation not found: missing
+%(root)s/images.txt:20: WARNING: no matching candidate for image URI u'foo.\\*'
 """
 
 if PY3:
     LATEX_WARNINGS = remove_unicode_literals(LATEX_WARNINGS)
 
 
-@with_app(buildername='latex')
+@with_app(buildername='latex', freshenv=True)  # use freshenv to check warnings
 def test_latex(app, status, warning):
     LaTeXTranslator.ignore_missing_images = True
     app.builder.build_all()
@@ -94,7 +91,7 @@ def test_latex(app, status, warning):
         os.chdir(cwd)
 
 
-@with_app(buildername='latex',
+@with_app(buildername='latex', freshenv=True,  # use freshenv to check warnings
           confoverrides={'latex_documents': [
               ('contents', 'SphinxTests.tex', 'Sphinx Tests Documentation',
                'Georg Brandl \\and someone else', 'howto'),
@@ -268,6 +265,25 @@ def test_numref_with_language_ja(app, status, warning):
     assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
 
 
+@with_app(buildername='latex', testroot='numfig',
+          confoverrides={'numfig': True, 'language': 'ru', 'latex_elements': {'babel': ''}})
+def test_numref_on_bable_disabled(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\renewcommand{\\figurename}{Fig. }' in result
+    assert '\\renewcommand{\\tablename}{Table }' in result
+    assert '\\SetupFloatingEnvironment{literal-block}{name=Listing }' in result
+    assert '\\hyperref[index:fig1]{Fig. \\ref{index:fig1}}' in result
+    assert '\\hyperref[baz:fig22]{Figure\\ref{baz:fig22}}' in result
+    assert '\\hyperref[index:table-1]{Table \\ref{index:table-1}}' in result
+    assert '\\hyperref[baz:table22]{Table:\\ref{baz:table22}}' in result
+    assert '\\hyperref[index:code-1]{Listing \\ref{index:code-1}}' in result
+    assert '\\hyperref[baz:code22]{Code-\\ref{baz:code22}}' in result
+
+
 @with_app(buildername='latex')
 def test_latex_add_latex_package(app, status, warning):
     app.add_latex_package('foo')
@@ -354,6 +370,15 @@ def test_latex_show_urls_is_inline(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
+    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
+    assert 'Auto footnote number \\footnote[1]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
+            'with a reference to }}}' in result)
     assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
     assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx} (http://sphinx-doc.org/)' in result
@@ -380,20 +405,29 @@ def test_latex_show_urls_is_footnote(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
-    assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
+    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
+    assert 'Auto footnote number \\footnote[2]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
+            'with a reference to }}}' in result)
+    assert 'First footnote: \\footnote[3]{\nFirst\n}' in result
     assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
     assert ('\\href{http://sphinx-doc.org/}{Sphinx}'
-            '\\footnote[3]{\nhttp://sphinx-doc.org/\n}' in result)
-    assert 'Third footnote: \\footnote[5]{\nThird\n}' in result
+            '\\footnote[4]{\nhttp://sphinx-doc.org/\n}' in result)
+    assert 'Third footnote: \\footnote[6]{\nThird\n}' in result
     assert ('\\href{http://sphinx-doc.org/~test/}{URL including tilde}'
-            '\\footnote[4]{\nhttp://sphinx-doc.org/\\textasciitilde{}test/\n}' in result)
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}\\protect\\footnotemark[7]}] '
-            '\\leavevmode\\footnotetext[7]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
-    assert ('\\item[{Footnote in term \\protect\\footnotemark[9]}] '
-            '\\leavevmode\\footnotetext[9]{\nFootnote in term\n}\nDescription' in result)
-    assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}\\protect'
-            '\\footnotemark[8]}] '
+            '\\footnote[5]{\nhttp://sphinx-doc.org/\\textasciitilde{}test/\n}' in result)
+    assert ('\\item[{\\href{http://sphinx-doc.org/}{URL in term}\\protect\\footnotemark[8]}] '
             '\\leavevmode\\footnotetext[8]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
+    assert ('\\item[{Footnote in term \\protect\\footnotemark[10]}] '
+            '\\leavevmode\\footnotetext[10]{\nFootnote in term\n}\nDescription' in result)
+    assert ('\\item[{\\href{http://sphinx-doc.org/}{Term in deflist}\\protect'
+            '\\footnotemark[9]}] '
+            '\\leavevmode\\footnotetext[9]{\nhttp://sphinx-doc.org/\n}\nDescription' in result)
     assert ('\\href{https://github.com/sphinx-doc/sphinx}'
             '{https://github.com/sphinx-doc/sphinx}\n' in result)
     assert ('\\href{mailto:sphinx-dev@googlegroups.com}'
@@ -408,6 +442,15 @@ def test_latex_show_urls_is_no(app, status, warning):
     print(result)
     print(status.getvalue())
     print(warning.getvalue())
+    assert 'Same footnote number \\footnote[1]{\nfootnote in bar\n} in bar.rst' in result
+    assert 'Auto footnote number \\footnote[1]{\nfootnote in baz\n} in baz.rst' in result
+    assert ('\\phantomsection\\label{index:id26}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to\\string-authoryear]'
+            '{\\emph{The section with a reference to \\phantomsection\\label{index:id1}'
+            '{\\hyperref[index:authoryear]{\\emph{{[}AuthorYear{]}}}}}}}' in result)
+    assert ('\\phantomsection\\label{index:id27}{\\hyperref[index:the\\string-section'
+            '\\string-with\\string-a\\string-reference\\string-to]{\\emph{The section '
+            'with a reference to }}}' in result)
     assert 'First footnote: \\footnote[2]{\nFirst\n}' in result
     assert 'Second footnote: \\footnote[1]{\nSecond\n}' in result
     assert '\\href{http://sphinx-doc.org/}{Sphinx}' in result
@@ -475,3 +518,25 @@ def test_toctree_maxdepth_howto(app, status, warning):
     print(status.getvalue())
     print(warning.getvalue())
     assert '\\setcounter{tocdepth}{2}' in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'master_doc': 'foo'})
+def test_toctree_not_found(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\setcounter{tocdepth}' not in result
+
+
+@with_app(buildername='latex', testroot='toctree-maxdepth',
+          confoverrides={'master_doc': 'bar'})
+def test_toctree_without_maxdepth(app, status, warning):
+    app.builder.build_all()
+    result = (app.outdir / 'Python.tex').text(encoding='utf8')
+    print(result)
+    print(status.getvalue())
+    print(warning.getvalue())
+    assert '\\setcounter{tocdepth}' not in result
